<?php namespace App;

use Illuminate\Database\Eloquent\Model;
use DateTime;
use Illuminate\Support\Facades\Auth;
use Jenssegers\Date\Date;
use Carbon\Carbon;


class Order extends Model {

	protected $table = 'orders';
    protected $primaryKey = 'order_id';
    protected $fillable = ['start_date','end_date'];

    public function product()
    {
        return $this->belongsTo('App\Product', 'product_id');
    }

    public function user()
    {
        return $this->belongsTo('App\User', 'user_id');
    }

    public function day()
    {
        return $this->hasMany('App\Day');
    }


    public static function createNewOrder($startDate, $endDate, $productID, $currentDateTime)
    {

        $newOrder = new Order;
        $newOrder->start_date = $startDate;
        $newOrder->end_date = $endDate;
        $newOrder->order_date = $currentDateTime;
        $product = Product::find($productID);
        $user = Auth::user();
        $newOrder = $product->order()->save($newOrder);
        $newOrder = $user->order()->save($newOrder);

        /**
         * According to order create single days
         */

        for ($i = $startDate; $i <= $endDate; $i++)
        {
            $newDay = new Day();
            $newDay->date = $i;
            $newDay->first_meal = true;
            $newDay->second_meal = true;
            $newDay->third_meal = true;
            $newDay->fourth_meal = true;
            $newDay->fifth_meal = true;
            $newDay = $newOrder->day()->save($newDay);
        }


        return $newOrder;
    }


    /**
     * Check the date format.
     *
     * @param $date
     * @param string $format
     * @return bool
     */
    public static function validateDate($date, $format = 'Y-m-d')
    {
        $d = DateTime::createFromFormat($format, $date);
        return $d && $d->format($format) == $date;
    }


    /**
     * This is a function to check the business rules.
     *
     * @param $startDate
     * @return bool
     */
    public static function orderCondition($startDate)
    {
        $currentDateTime = Carbon::now();

        // Re-create start day format to "Y-m-d 12:00:00".
        $date = Carbon::createFromFormat('Y-m-d', $startDate)->setTime(0,0,0)->addHours(12);

        // Allow client make an order only to 12 o'clock.
        if ( $date->subDay(1) > $currentDateTime ) {
            return true;
        } else {
            return false;
        }
    }

    public static function editCondition($startDate)
    {
        $currentDateTime = Carbon::now();

        // Re-create start day format to "Y-m-d 12:00:00".
        $date = Carbon::createFromFormat('Y-m-d H:i:s', $startDate)->setTime(0,0,0)->addHours(12);

        // Allow client make an order only to 12 o'clock.
        if ( $date->subDay(1) > $currentDateTime ) {
            return true;
        } else {
            return false;
        }
    }



    public static function isOrderActual($order)
    {
        $currentDateTime = Carbon::now();
        $days = $order->day->all();

        $result = false;

        foreach ($days as $day) {
            if ($day->date >= $currentDateTime) {
                $result = true;
            }
        }


        return $result;
    }


    /**
     * Get nice czech date format.
     *
     * @param $date
     * @return string
     */
    public function getCzechDate($date)
    {
        $date = new Date($date);
        $czechDate = $date->format('j. F Y');
        return $czechDate;
    }

    /**
     * Get numeric date format.
     *
     * @param $date
     * @return Date|string
     */
    public function getNumericDate($date)
    {
        $date = new Date($date);
        $date = $date->format('j.n. Y');
        return $date;
    }

    /**
     * Find a product which is included in order.
     *
     * @param $order
     * @return mixed
     */
    public function getProductForOrder($order)
    {
        $product = $order->product()->first();
        return $product;
    }

    /**
     * Get all days included in order.
     *
     * @param $order
     * @return mixed
     */
    public function getDaysForOrder($order)
    {
        $days = Order::find($order->order_id)->day;
        return $days;
    }

}
